Your task is to create table that contains the following content:
{{ content }}

Let's say you have access to the following database (tables and their schemata).
{{ tables }}

Here are the requested columns. Which ones can you generate?
{{ schema }}

Return only a comma separated string with the columns you can generate using information from the given tables. Use
the original requested column names in your response.