Your task is to create table that contains the following content:
{{ content }}

Which of the following columns can you generate with your own information?
{{ schema }}

Return only a comma separated string. Use the original column names provided above. If you cannot return anything
return an empty string.