# Copyright (c) 2025 Victor Giannakouris
#
# This file is part of SwellDB and is licensed under the MIT License.
# See the LICENSE file in the project root for more information.


class Globals:
    """
    A class to hold global variables for the application.
    """

    # Global variables
    OPENAI_API_KEY: str = "openai_api_key"
    GOOGLE_SERPER_API_KEY: str = "serper_api_key"
