Generate a list of search engine queries that you need to construct a table of the following content and schema:

content: {{ prompt }}

schema: {{ schema }}

{% if data %}
Your response should contain information about the following data:
data: {{ data }}

The queries should be splittable by a breakline character. Generate a query for each record. If you reuse any of the
provided data, do not alternate their values. Use the original ones.

Do not include any quotes. Return only the list of the queries and nothing else. 