# Copyright (c) 2025 Victor Giannakouris
#
# This file is part of SwellDB and is licensed under the MIT License.
# See the LICENSE file in the project root for more information.


class Datasource:
    def __init__(self):
        pass

    def search(self, query: str):
        pass
